import '../src/validators_library_validators.dart';

import '../src/helper.dart';
import '../src/value_exceptions.dart';
import '../src/value_object.dart';

enum IPVersion { v4, v6 }

class IPAddress extends ValueObject<String> {
  /// Default value: `ip`
  static String key = 'ip';

  final IPVersion version;

  /// One of the following [ValueException] maybe thrown:
  /// - RequiredValueException
  /// - InvalidValueException
  factory IPAddress(String value) {
    if (value.isEmpty) {
      throw const RequiredValueException();
    } else if (isIP(value, '4')) {
      return IPAddress._(value, IPVersion.v4);
    } else if (isIP(value, '6')) {
      return IPAddress._(value, IPVersion.v6);
    }

    throw InvalidValueException(value, message: 'Invalid IP address.');
  }

  const IPAddress._(String value, this.version) : super(value);

  /// [map] can be a `Map`, `String` or `ValueObject`
  ///
  /// One of the following [ValueException] maybe thrown:
  /// - RequiredValueException
  /// - InvalidValueException
  factory IPAddress.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return IPAddress(flatMap[key]);
    }
    return IPAddress(map.toString());
  }

  Map<String, String> toJson() => {key: toString()};
}
