import '../src/helper.dart';
import '../src/validators_library_validators.dart';
import '../src/value_exceptions.dart';
import '../src/value_object.dart';

enum ISBNVersion { v10, v13 }

class ISBN extends ValueObject<String> {
  /// Default value: `isbn`
  static String key = 'isbn';

  final ISBNVersion version;

  /// One of the following [ValueException] maybe thrown:
  /// - RequiredValueException
  /// - InvalidValueException
  factory ISBN(String value) {
    if (value.isEmpty) {
      throw const RequiredValueException();
    } else if (isISBN(value, '10')) {
      return ISBN._(value, ISBNVersion.v10);
    } else if (isISBN(value, '13')) {
      return ISBN._(value, ISBNVersion.v13);
    }
    throw InvalidValueException(value, message: 'Invalid ISBN number.');
  }

  const ISBN._(String value, this.version) : super(value);

  /// [map] can be a `Map`, `String` or `ValueObject`
  ///
  /// One of the following [ValueException] maybe thrown:
  /// - RequiredValueException
  /// - InvalidValueException
  factory ISBN.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return ISBN(flatMap[key]);
    }
    return ISBN(map.toString());
  }

  Map<String, String> toJson() => {key: toString()};
}
