import 'package:test/test.dart';
import 'package:valid_value_objects/src/boundary_date.dart';
import 'package:valid_value_objects/src/value_exceptions.dart';

void main() {
  test('Invalid boundaries', () {
    expect(
      () => BoundaryDateTime(lowerBound: null, upperBound: null),
      throwsA(isA<RequiredValueException>()),
    );

    expect(
      () => BoundaryDateTime(
          lowerBound: DateTime(2012), upperBound: DateTime(2010)),
      throwsA(isA<InvalidValueException>()),
    );
  });
  test('Upper and lowerbound', () {
    final bd1 = BoundaryDateTime(
        lowerBound: DateTime(2010), upperBound: DateTime(2011));

    expect(bd1.boundary(DateTime(2010, 10)), Boundary.inRange);
    expect(bd1.boundary(DateTime(2009)), Boundary.before);
    expect(bd1.boundary(DateTime(2012)), Boundary.after);
  });

  test('Lowerbound only', () {
    final bd1 = BoundaryDateTime(lowerBound: DateTime(2010));

    expect(bd1.boundary(DateTime(2010, 10)), Boundary.inRange);
    expect(bd1.boundary(DateTime(2009)), Boundary.before);
    expect(bd1.boundary(DateTime(2012)), Boundary.inRange);
  });

  test('Upperbound only', () {
    final bd1 = BoundaryDateTime(upperBound: DateTime(2011));

    expect(bd1.boundary(DateTime(2010, 10)), Boundary.inRange);
    expect(bd1.boundary(DateTime(2010)), Boundary.inRange);
    expect(bd1.boundary(DateTime(2012)), Boundary.after);
  });
}
